/**
 * Copyright (c) IBM Japan 2013. All rights reserved.
 */
package net.dependableos.dcase.toolchain.rhapsodyextension;

import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import net.dependableos.dcase.toolchain.rhapsodyextension.commands.SysML_ExportDCase;
import net.dependableos.dcase.toolchain.rhapsodyextension.commands.SysML_ImportDCase;
import net.dependableos.dcase.toolchain.rhapsodyextension.commands.SysML_LinkCreator;

import com.telelogic.rhapsody.core.*;

/**
 * D-CasevOC̃CNX
 */
public class DCasePlugin extends RPUserPlugin {
	
	/** Õ^Cg */
	public static final String LOG_TITLE = "D-Case Plugin";
	
	/** ContextN쐬̃j[ */
	public static final String MENU_LABEL_LINK_CONTEXT = "D-Case\\Create Link to SysML Model Element\\Context";
	
	/** AssumptionN쐬̃j[ */
	public static final String MENU_LABEL_LINK_ASSUMPTION = "D-Case\\Create Link to SysML Model Element\\Assumption";
	
	/** ValidationN쐬̃j[ */
	public static final String MENU_LABEL_LINK_JUSTIFICATION = "D-Case\\Create Link to SysML Model Element\\Justification";
	
	/** C|[g̃j[ */
	public static final String MENU_LABEL_IMPORT = "D-Case\\Import";
	
	/** GNX|[g̃j[ */
	public static final String MENU_LABEL_EXPORT = "D-Case\\Export";
	
	/** Rhapsody Application */
	protected IRPApplication app = null;
	
	
	/**
	 * called when the plug-in is loaded
	 */
	public void RhpPluginInit(final IRPApplication rpyApplication) {
		this.app = rpyApplication;
		
		app.writeToOutputWindow(LOG_TITLE, "D-Case Plugin (ver. 0.2) is loaded.\n");
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
	}

	/**
	 * called when the plug-in menu item under the "Tools" menu is selected	
	 */
	public void RhpPluginInvokeItem() {
		// do nothing
//		app.writeToOutputWindow(LOG_TITLE, "Menu is selected.\n");
	}

	/**
	 * called when the plug-in pop-up menu (if applicable) is selected
	 */
	public void OnMenuItemSelect(String menuItem) {
		try{
		//show the selected element name
		String selElemName = "";
		IRPModelElement element = app.getSelectedElement();
		if(element != null){
			selElemName = element.getName();			
		} else {
			selElemName = "No selected element";
		}
		String msg = selElemName + " is selected by " + menuItem + ".\n"; 
//		app.writeToOutputWindow(LOG_TITLE , msg);
		System.out.println(msg);
		
		// N
		if ( menuItem.equals(MENU_LABEL_LINK_CONTEXT) ||
				menuItem.equals(MENU_LABEL_LINK_ASSUMPTION) ||
				menuItem.equals(MENU_LABEL_LINK_JUSTIFICATION) ) {
			
			SysML_LinkCreator creator = new SysML_LinkCreator(app);
			creator.createLink(menuItem);
		}
		
		// C|[g
		else if ( menuItem.equals(MENU_LABEL_IMPORT) ) {
			SysML_ImportDCase importer = new SysML_ImportDCase(app);
			importer.importDCase();
		}
		
		// GNX|[g
		else if ( menuItem.equals(MENU_LABEL_EXPORT) ) {
			SysML_ExportDCase exporter = new SysML_ExportDCase(app);
			exporter.exportDCase();
		}
		
		else {
			app.writeToOutputWindow(LOG_TITLE , msg);
		}
		
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	/**
	 * called when the plug-in popup trigger (if applicable) fired
	 */
	public void OnTrigger(String trigger) {
		// do nothing
//		app.writeToOutputWindow(LOG_TITLE , "Popup trigger is fired.\n");
	}

	/**
	 * called when the project is closed - if true is returned the plugin will be unloaded
	 */
	public boolean RhpPluginCleanup() {
		//cleanup
		app = null;
		
		//return true so the plug-in will be unloaded now (on project close)
		return true;
	}

	/**
	 * called when Rhapsody exits
	 */
	public void RhpPluginFinalCleanup() {
		// do nothing
	}
}
