/**
 * Copyright (c) IBM Japan 2013. All rights reserved.
 */
package net.dependableos.dcase.toolchain.rhapsodyextension.model;

import java.util.StringTokenizer;

import com.telelogic.rhapsody.core.*;

/**
 * D-Casẽfvf̂Af}ǗNX
 */
public class DCaseDiagram {
	
	/** Namel */
	protected String name; // ex. d-case diagram_0
	
	/** Display Namel */
	protected String displayName; // ex. ACCSD-Case
	
	/** Base Class */
	protected String metaClassBase; // ex. ObjectModelDiagram
	
	/** Stereotype */
	protected String metaClass; // ex. D-Case Diagram
	
	/** bp[Ώۂ̃fvf */
	protected IRPObjectModelDiagram element;
	
	/**
	 * D-Casefvf̂f}̃RXgN^ <br>
	 * ̃fꗗw肵fvfÃfvf΁ÃfvfbvB
	 * ̃fvfȂ΁AfvfVK쐬ăbvB
	 * @param name@fvf̖
	 * @param displayName fvf̕\
	 * @param prj fvf܂ރvWFNg
	 * @param pkg fvf܂ރpbP[W
	 */
	public DCaseDiagram(String name, String displayName, IRPProject prj, IRPPackage pkg){
		this.name = name;
		this.displayName = displayName;
		this.metaClassBase = "ObjectModelDiagram";
		this.metaClass = "D-Case Diagram";
		
		// fvfȂΒǉ
		this.element = (IRPObjectModelDiagram) pkg.findNestedElementRecursive(name, metaClass);
		if(element == null){
			element = pkg.addObjectModelDiagram(name);
			element.setDisplayName(displayName);
			element.addStereotype(metaClass, metaClassBase);
		}
		
	}
	
	/**
	 * D-Casefvf̂f}̃RXgN^ <br>
	 * ̃fꗗw肵fvfÃfvf΁ÃfvfbvB
	 * ̃fvfȂ΁AfvfVK쐬ăbvB
	 * @param element Ώۂ̃fvf
	 */
	public DCaseDiagram(IRPDiagram element){
		this.name = element.getName();
		this.displayName = element.getDisplayName();
		this.metaClassBase = "ObjectModelDiagram";
		this.metaClass = "D-Case Diagram";
		
		if (element instanceof IRPObjectModelDiagram) {
			this.element = (IRPObjectModelDiagram) element;
		}
		else {
			System.err.println("Error DCaseDiagram : Type mismatch " + element.getClass());
			
			// fvfȂΒǉ
//			this.element = (IRPObjectModelDiagram) prj.findNestedElementRecursive(name, metaClass);
//			if(this.element == null){
//				this.element = prj.addObjectModelDiagram(name);
//				this.element.setDisplayName(displayName);
//				this.element.addStereotype(metaClass, metaClassBase);
//			}
		}
	}
	
	/**
	 * m[h`D-Casefvfw肵ʒuɒǉÃOtm[hԂ
	 * @param nTrg ǉΏۂD-Casem[h̃fGg
	 * @param xTrgPosition ǉΏۂD-Casem[h̍[xW
	 * @param yTrgPosition ǉΏۂD-Casem[h̍[yW
	 * @param trgWidth ǉΏۂD-Casem[h̕
	 * @param trgHeight ǉΏۂD-Casem[h̍
	 * @return ǉD-Casem[h̃Otm[h
	 */
	public IRPGraphNode addGraphNode(DNode nTrg, int xTrgPosition, int yTrgPosition, int trgWidth, int trgHeight){
		IRPGraphNode gTrg = element.addNewNodeForElement(nTrg.getElement(), xTrgPosition, yTrgPosition, trgWidth, trgHeight);
		nTrg.getElement().setIsShowDisplayName(1);
		return gTrg;
	}
	
	/**
	 * GbW`D-Casefvfw肵ʒuɒǉÃOtm[hԂ<br>
	 * w肷D-CaseGbW̍W͊D-Casem[h̓łKv
	 * @param eTrg ǉΏۂD-CaseGbW̃fGg
	 * @param gSrc D-Casem[h(ˑ)̃Otm[h
	 * @param xSrcPosition ǉΏۂD-CaseGbW(ˑ)xW
	 * @param ySrcPosition ǉΏۂD-CaseGbW(ˑ)yW
	 * @param gTrg D-Casem[h(ˑ)̃Otm[h
	 * @param xTrgPosition ǉΏۂD-CaseGbW(ˑ)xW
	 * @param yTrgPosition ǉΏۂD-CaseGbW(ˑ)yW
	 * @return ǉD-Casefvf̃OtGbW
	 */
	public IRPGraphEdge addGraphEdge(DEdge eTrg, IRPGraphNode gSrc, int xSrcPosition, int ySrcPosition, IRPGraphNode gTrg, int xTrgPosition, int yTrgPosition){
		IRPGraphEdge edge = element.addNewEdgeForElement(eTrg.getElement(), gSrc, xSrcPosition, ySrcPosition, gTrg, xTrgPosition, yTrgPosition);
		return edge;
	}
	
	/**
	 * GbW`D-Casefvfw肵ʒuɒǉÃOtm[hԂ
	 * @param eTrg ǉΏۂD-CaseGbW̃fGg
	 * @param gSrc D-Casem[h(ˑ)̃Otm[h
	 * @param gTrg D-Casem[h(ˑ)̃Otm[h
	 * @return ǉD-Casefvf̃OtGbW
	 */
	public IRPGraphEdge addGraphEdge(DEdge eTrg, IRPGraphNode gSrc, IRPGraphNode gTrg){
		// ʒu擾
		int xSrcPosition = 0;
		int ySrcPosition = 0;
		
		String xySrcPosition = gSrc.getGraphicalProperty("Position").getValue();
		StringTokenizer st = new StringTokenizer(xySrcPosition, ",");
		if(st.hasMoreTokens()) {
			xSrcPosition = Integer.parseInt(st.nextToken());
		}
		if(st.hasMoreTokens()) {
			ySrcPosition = Integer.parseInt(st.nextToken());
		}
		int srcWidth  = Integer.parseInt(gSrc.getGraphicalProperty("Width").getValue());
		int srcHeight = Integer.parseInt(gSrc.getGraphicalProperty("Height").getValue());
		
		int xTrgPosition = 0;
		int yTrgPosition = 0;
		
		String xyTrgPosition = gTrg.getGraphicalProperty("Position").getValue();
		st = new StringTokenizer(xyTrgPosition, ",");
		if(st.hasMoreTokens()) {
			xTrgPosition = Integer.parseInt(st.nextToken());
		}
		if(st.hasMoreTokens()) {
			yTrgPosition = Integer.parseInt(st.nextToken());
		}
		int trgWidth  = Integer.parseInt(gTrg.getGraphicalProperty("Width").getValue());
		int trgHeight = Integer.parseInt(gTrg.getGraphicalProperty("Height").getValue());
		
		// ֌W𔠂̒S߂̃ItZbg
		int xSrcOffset = srcWidth / 2;
		int ySrcOffset = srcHeight / 2;
		int xTrgOffset = trgWidth / 2;
		int yTrgOffset = trgHeight / 2;
		
		IRPGraphEdge edge = addGraphEdge(eTrg, 
				gSrc, xSrcPosition + xSrcOffset, ySrcPosition + ySrcOffset, 
				gTrg, xTrgPosition + xTrgOffset, yTrgPosition + yTrgOffset);
		return edge;
	}
	
	/**
	 * w肵D-CasefvfD-Casem[hD-CaseGbWǉÃOtm[hԂ
	 * @param nTrg ǉΏۂD-Casem[h̃fGg
	 * @param xTrgPosition ǉΏۂD-Casem[h̍[xW
	 * @param yTrgPosition ǉΏۂD-Casem[h̍[yW
	 * @param trgWidth ǉΏۂD-Casem[h̕
	 * @param trgHeight ǉΏۂD-Casem[h̍
	 * @param eTrg ǉΏۂD-CaseGbW̃fGg
	 * @param gSrc D-Casem[h̃Otm[h
	 * @return ǉD-Casem[h̃Otm[h
	 */
	public IRPGraphNode addGraphNodeWithEdge(DNode nTrg, int xTrgPosition, int yTrgPosition, int trgWidth, int trgHeight, DEdge eTrg, IRPGraphNode gSrc) {
		// ʒu擾
		int xSrcPosition = 0;
		int ySrcPosition = 0;
		
		String xySrcPosition = gSrc.getGraphicalProperty("Position").getValue();
		StringTokenizer st = new StringTokenizer(xySrcPosition, ",");
		if(st.hasMoreTokens()) {
			xSrcPosition = Integer.parseInt(st.nextToken());
		}
		if(st.hasMoreTokens()) {
			ySrcPosition = Integer.parseInt(st.nextToken());
		}
		int srcWidth  = Integer.parseInt(gSrc.getGraphicalProperty("Width").getValue());
		int srcHeight = Integer.parseInt(gSrc.getGraphicalProperty("Height").getValue());
		
		// ֌W𔠂̒S߂̃ItZbg
		int xSrcOffset = srcWidth / 2;
		int ySrcOffset = srcHeight / 2;
		int xTrgOffset = trgWidth / 2;
		int yTrgOffset = trgHeight / 2;
		
//		IRPGraphNode gTrg = element.addNewNodeForElement(nTrg.getElement(), xTrgPosition, yTrgPosition, trgWidth, trgHeight);
		IRPGraphNode gTrg = addGraphNode(nTrg, xTrgPosition, yTrgPosition, trgWidth, trgHeight);
		addGraphEdge(eTrg, gSrc, xSrcPosition + xSrcOffset, ySrcPosition + ySrcOffset, gTrg, xTrgPosition + xTrgOffset, yTrgPosition + yTrgOffset);
		return gTrg;
	}
	
	/**
	 * D-Case}J
	 */
	public void openDiagram() {
		element.openDiagram();
	}

	/**
	 * eXg
	 */
	public static void main(String[] args){
		IRPApplication app = null;
		try{
			app = RhapsodyAppServer.getActiveRhapsodyApplication();
		}
		catch(Exception e){
			System.out.println("Rhapsody is not active.");
			e.printStackTrace();
			System.exit(0);
		}
		IRPProject prj = app.activeProject();
		IRPPackage pkg = (IRPPackage) prj.findNestedElement("Default", "Package");
		if(pkg == null){
			pkg = prj.addPackage("Default");
		}
		
		DCaseDiagram d = new DCaseDiagram("TestDCase", "eXgpD-Case Diagram", prj, pkg);
		DGoal nG = new DGoal("TestG", "ACC͈S", prj, pkg);
		DContext nC = new DContext("TestC", "ꗗ", prj, pkg);
		DStrategy nS = new DStrategy("TestS", "Ƃɋc_", prj, pkg);

		DInContextOf eC = new DInContextOf("In_Context_Of_1", "In Context Of 1", prj, pkg, nG, nC);
		DSupportedBy eS = new DSupportedBy("Supported_By_1", "Supported By 1", prj, pkg, nG, nS);
		
		IRPGraphNode gG = d.addGraphNode(nG, 100, 200, 80, 60);
		IRPGraphNode gC = d.addGraphNodeWithEdge(nC, 300, 200, 80, 60, eC, gG);
		IRPGraphNode gS = d.addGraphNodeWithEdge(nS, 100, 400, 80, 60, eS, gG);
	}
}
